@@ -1,6 +1,8 @@
 /****************************************************************************
  * apps/examples/usrsocktest/usrsocktest_block_recv.c
  *
+ * SPDX-License-Identifier: Apache-2.0
+ *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.  The
@@ -558,7 +560,7 @@ TEST_SETUP(block_recv)
 
 TEST_TEAR_DOWN(block_recv)
 {
-  int ret;
+  int unused_data ret;
 
   if (sd >= 0)
     {
