@@ -33,6 +33,14 @@
  * Public Types
  ****************************************************************************/
 
+#ifdef __cplusplus
+#define EXTERN extern "C"
+extern "C"
+{
+#else
+#define EXTERN extern
+#endif
+
 /* Nxscope frame handler */
 
 struct nxscope_frame_s
@@ -104,4 +112,9 @@ int nxscope_proto_ser_init(FAR struct nxscope_proto_s *proto, FAR void *cfg);
 void nxscope_proto_ser_deinit(FAR struct nxscope_proto_s *proto);
 #endif
 
+#undef EXTERN
+#ifdef __cplusplus
+}
+#endif
+
 #endif  /* __APPS_INCLUDE_LOGGING_NXSCOPE_NXSCOPE_PROTO_H */
