@@ -1,14 +1,10 @@
 /****************************************************************************
  * apps/wireless/wapi/src/wapi.c
  *
- *   Copyright (C) 2011, 2017, 2019 Gregory Nutt. All rights reserved.
- *   Author: Gregory Nutt <gnutt@nuttx.org>
- *
- * Largely and original work, but highly influenced by sampled code provided
- * with WAPI:
- *
- *   Copyright (c) 2010, Volkan YAZICI <volkan.yazici@gmail.com>
- *   All rights reserved.
+ * SPDX-License-Identifier: BSD-2-Clause
+ * SPDX-FileCopyrightText: 2011,2017,2019 Gregory Nutt. All rights reserved.
+ * SPDX-FileCopyrightText: 2010 Volkan YAZICI <volkan.yazici@gmail.com>
+ * SPDX-FileContributor: Gregory Nutt <gnutt@nuttx.org>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -911,6 +907,7 @@ static int wapi_country_cmd(int sock, int argc, FAR char **argv)
 
   if (argc == 1)
     {
+      memset(country, 0, sizeof(country));
       ret = wapi_get_country(sock, argv[0], country);
       if (ret >= 0)
         {
